program reperes_geodesic;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  donnee, outils;

type

  TLocalisation = (tlMetropole, tlOutreMer);

  TFiche = record
    Nom : string;
    numero : string;
    Lieu_dit : string;
    Reseau : string;
    Nb_Point : integer;
    Points : array[1..30] of integer;
    X, Y : string;
    Localisation: TLocalisation;
  end;

const

  Separateur : char ='$';
  Max_Fiche = 100000;
  Max_Correction = 1000;
  BaseURL : string = 'http://ancien-geodesie.ign.fr/';


var

  Les_Fiches : array[0..Max_Fiche] of TFiche;
  Corrections : array[0..Max_Correction, 0..1] of string;
  F, G, OSM : TextFile;
  S, T : string;
  Nb_Correction, Nb_Fiche, Fiche_Courante, Point_Courant, I, J : integer;
  Site, Indice, Designation, Remarque, Altitude, Lat, Long, Proj_Local : string;
  Coord_Ok : boolean;
  XDest, YDest : string;

procedure Lit_Corrections;
begin
  assignFile(F, 'corrections.txt');
  reset(F);
  try
    Nb_Correction := 0;
    while not Eof(F) do
    begin
      readln(F, S);
      delete(S,1,1); // suppression du ; au dbut
      Corrections[Nb_Correction, 0] := copy(S,1, pos(':',S) - 1);
      Corrections[Nb_Correction, 1] := copy(S, pos(':',S) + 1, length(S));
      inc(Nb_Correction);
    end;
  finally
    CloseFile(F);
  end;
  Corrections[Nb_Correction, 0] := 'cass? e';
  Corrections[Nb_Correction, 1] := 'cassée';
  inc(Nb_Correction);
end;

procedure Corrige_Accents(var Chaine : string);
var
  I, J : integer;
begin
  for I:=0 to Nb_Correction - 1 do
    while pos(Corrections[I, 0], Chaine)<>0 do
    begin
      J := pos(Corrections[I, 0], Chaine);
      delete(Chaine, J, length(Corrections[I, 0]));
      insert(Corrections[I, 1], Chaine, J);
    end;

// convertit aussi les &
  for I:=length(Chaine) downto 1 do
    if Chaine[I]='&' then
      insert('amp;', Chaine, I + 1);

// convertit aussi les apostrophes
  while pos('''', Chaine)<>0 do
  begin
    insert('&apos;', Chaine, pos('''',Chaine)+1);
    delete(Chaine, pos('''',Chaine), 1);
  end;
end;

procedure Espace_Point(var Chaine : string);
// supprime les espaces et remplace la virgule par un point
begin
  while pos(' ',Chaine)<>0 do
    delete(Chaine, pos(' ', Chaine), 1);
  if pos(',',Chaine)<>0 then
    Chaine[pos(',',Chaine)]:='.';
end;

function GetNext : string;
begin
  result := copy(S,1, pos(Separateur, S) -1);
  delete(S,1,pos(Separateur,S));
end;

begin
  Lit_Corrections;
// lecture des fiches
  assignFile(F,'fiche.csv');
  reset(F);
  try
    Nb_Fiche := 0;
    while not Eof(F) do
    begin
      inc(Nb_Fiche);
      readln(F, S);
      GetNext; // saute FICHE
      with Les_Fiches[Nb_Fiche-1] do
      begin
        Nb_Point := 0;
        Numero := GetNext;
        Nom := GetNext; Corrige_Accents(Nom);
        GetNext; // saute Rseau Godsique Franais
        T := GetNext; //rseau
        if pos('site NTF d''ordre ',T)=1 then
          Reseau := 'NTF-'+T[18]
        else if T='site RBF' then
          Reseau := 'RBF'
        else if T='' then
          Reseau := ''
        else
          raise Exception.Create('Rseau inconnu');
        GetNext; // saute Dpartement
        GetNext; // saute Numro Dpartement
        GetNext; // saute Commune
        Lieu_dit := GetNext; Corrige_Accents(Lieu_dit);
      end;

    end;
  finally
    closeFile(F);
  end;

// lecture des repres et criture de ceux-ci dans le fichier OSM

  Donnees:=TDonnees.Create;
  try
    Donnees.AjoutLigne('Projection "Lambert Conique Scant"="LT 93" "Lambert-93" "WGS84" 700000 6600000 3 4630'' 44 49');
    Donnees.AjoutLigne('Ellipsoide="Clarke80" "Clarke 1880" 6378249.2 0.0824832568');
    Donnees.AjoutLigne('Datum="NTF" "Nouvelle Triangulation Franaise" "Clarke80" -168 -60 320 gr3df97a.txt');
    Donnees.AjoutLigne('Projection "Lambert Conique Tangent"="LT 1" "Lambert I" "NTF" 0.99987734 600000 200000 220''14.025'''' 55g');
    Donnees.AjoutLigne('Projection "Lambert Conique Tangent"="LT 2" "Lambert II" "NTF" 0.99987742 600000 200000 220''14.025'''' 52g');
    Donnees.AjoutLigne('Projection "Lambert Conique Tangent"="LT 3" "Lambert III" "NTF" 0.9998775 600000 200000 220''14.025'''' 49g');
    Donnees.AjoutLigne('Projection "Lambert Conique Tangent"="LT 4" "Lambert IV" "NTF" 0.99994471 234.358 185861.369 220''14.025'''' 46.85g');
    Donnees.AjoutLigne('Multiple "Transverse Mercator Multiple"="UTM" "Universal Transverse Mercator" "WGS84" 0.9996 500000 0 10000000 -177 6');

    Donnees.AjoutLigne('Ellipsoide="Hayford 24" "International Hayford 1924" 6378388 0.0819918900');
    Donnees.AjoutLigne('Datum="Ste-Anne" "Guadeloupe (Ste-Anne)" "Hayford 24" -472.29 -5.63 -304.12 0.4362 -0.8374 0.2563 1.8984');
    Donnees.AjoutLigne('Projection "UTM Fuseau"="Guadeloupe SA" "UTM20 Guadeloupe (Ste-Anne)" "Ste-Anne" 20 0');
    Donnees.AjoutLigne('Projection "UTM Fuseau"="UTM 22 N" "Zone Utm 22 Nord" "WGS84" 22 0');
//    Donnees.AjoutLigne('');


    assignFile(OSM, 'reperes_ign.osm');
    rewrite(OSM);

    try
      writeln(OSM,'<?xml version=''1.0'' encoding=''UTF-8''?>');
      writeln(OSM,'<osm version=''0.6'' generator=''reperes_geodesic Eric Sibert''>');
      Fiche_Courante := 0;
      assignFile(F,'repere.csv');
      reset(F);
      try
        assignFile(G,'fiche-repere.csv');
        reset(G);
        try

          Point_Courant := 0;
          while not Eof(F) do
          begin
            inc(Point_Courant);
            readln(F, S);
            GetNext; // saute REPERE
            Site := GetNext; // N de site (= fiche)
            // on suppose que les fiches et les points sont classs dans le mme ordre
            // et qu'il n'y a pas besoin de parcours toutes les fiches pour trouver celle
            // qui correspond au point mais juste d'aller  la (ou les) suivante(s)
            while (Site<>Les_Fiches[Fiche_Courante].numero) do
              inc(Fiche_Courante);
            with Les_Fiches[Fiche_Courante] do
            begin
              inc(Nb_Point);
              Points[Nb_Point] := Point_Courant;
            end;
            Indice := GetNext; 
            Designation := GetNext; Corrige_Accents(Designation);
            GetNext; // saute Rseau godsique franais
            Remarque := GetNext; Corrige_Accents(Remarque);
            GetNext; // saute photo
            GetNext; // saute Systme RGF 93
            GetNext; // saute mthode de calcul
            Long := GetNext; // longitude en RGF 93
            Lat := GetNext; // latitude en RGF93
            Coord_Ok := Lat<>'';
            if Coord_Ok then
            begin // convertir le texte RGF93 en texte dcimal  sparateur .
              Espace_Point(Lat);
              if Lat[length(Lat)]='S' then
                Lat := '-'+Lat;
              delete(Lat,length(Lat),1);
              Espace_Point(Long);
              if Long[length(Long)]='O' then
                Long := '-'+Long;
              delete(Long,length(Long),1);
              Donnees.Conversion('WGS84', 'WGS84', Lat, Long, 0, I, XDest, YDest, uaDegre);
              Lat := copy(XDest, 1, length(XDest)-1); // copie en supprimant 
              Long := copy(YDest, 1, length(YDest)-1); // copie en supprimant 
            end;
            GetNext; // saute hauteur GRS80
            GetNext; // saute 'vide'
            Proj_Local := GetNext; // projection locale (compatible WGS84)
            GetNext; // saute NGF69
            GetNext; // saute mthode de calcul
            if Coord_Ok then
            begin
              GetNext; // saute X Lambert 93
              GetNext; // saute Y Lambert 93
            end
            else
            begin
              Lat := GetNext; // X en Lambert 93 (ou UTM/WGS84 pour les DOM)
              Long := GetNext; // Y en Lambert 93 (ou UTM/WGS84 pour les DOM)
              Coord_Ok := Lat<>'';
              if Site='7155101' then // les coordonnes Lambert 93 du site HUCHON I sont fausses
                Coord_Ok := false; // aller chercher le Lambert Zone
              if Coord_Ok then
              begin // convertir le texte IGN Lt93 en RGF93(WGS84) texte dcimal  sparateur .
                Espace_Point(Lat);
                Espace_Point(Long);
                if Proj_Local = 'Système RGF93 - Projection LAMBERT - 93' then
                  Donnees.Conversion('LT 93', 'WGS84', Lat, Long, 0, I, XDest, YDest, uaDegre)
                else
                if Proj_Local = 'Projection : UTM NORD FUSEAU 20' then
                  Donnees.Conversion('UTM', 'WGS84', Lat, Long, 20, I, XDest, YDest, uaDegre)
                else
                if Proj_Local = 'Projection : UTM NORD FUSEAU 21' then
                  Donnees.Conversion('UTM', 'WGS84', Lat, Long, 21, I, XDest, YDest, uaDegre)
                else
                if Proj_Local = 'Projection : UTM NORD FUSEAU 22' then
                  Donnees.Conversion('UTM', 'WGS84', Lat, Long, 22, I, XDest, YDest, uaDegre)
                else
                  raise Exception.CreateFmt('Projection inconnue : %s',[Proj_Local]);
                Lat := copy(XDest, 1, length(XDest)-1); // copie en supprimant 
                Long := copy(YDest, 1, length(YDest)-1); // copie en supprimant 
              end;
            end;
            Altitude := GetNext;
            if not Coord_Ok then
            begin // toujours pas de coordonnes, on va chercher le Lambert Zone ou UTM local dans les DOM
              GetNext; // saute la prcision
              Proj_Local := GetNext; // zone Lambert
              GetNext; // saute 'vide'
              Lat := GetNext; // X en Lambert Zone ou UTM local dans les DOM
              Long := GetNext; // Y en Lambert Zone ou UTM local dans les DOM
              Coord_Ok := Lat<>'';
              if not Coord_Ok then
              begin  // fiche sans repre
                dec(Point_Courant);
                dec(Les_Fiches[Fiche_Courante].Nb_Point);
              end
              else
              begin
                Espace_Point(Lat);
                Espace_Point(Long);


                if copy(Proj_Local,1, length(Proj_Local)-1)='Système NTF - Projection LAMBERT ' then
                  Donnees.Conversion('LT '+Proj_Local[length(Proj_Local)], 'WGS84', Lat, Long, 0, I, XDest, YDest, uaDegre)
                else
                if Proj_Local = 'Système : GUADELOUPE - STE ANNE Projection : UTM NORD FUSEAU 20' then
                  Donnees.Conversion('Guadeloupe SA', 'WGS84', Lat, Long, 0, I, XDest, YDest, uaDegre)
                else
                  raise Exception.CreateFmt('Projection inconnue : %s',[Proj_Local]);
                Lat := copy(XDest, 1, length(XDest)-1); // copie en supprimant 
                Long := copy(YDest, 1, length(YDest)-1); // copie en supprimant 
              end;
            end;

            // lecture dans le fichier fiche-repere.csv  pour trouver l'url
            readln(G, S);
            GetNext; // saute REPERE
            GetNext; // saute n de site
            GetNext; // saute indice repre
            GetNext; // saute dsignation
            Corrige_Accents(S); // pour &

            if Coord_Ok then
            begin
              // il faut transmettre les coordonnes du point  la fiche pour pouvoir reconstituer
              // l'url de la fiche (a positionne l'extrait de carte au 1/50.000)
              // il faut dterminer si on est mtropole, en Outre-Mer
              // les TAAF semblent compatibles avec OM

              if copy(S, 1, 15)='fiche_point.asp' then // mtropole
              begin  // conversion Lambert 93
                Donnees.Conversion('WGS84', 'LT 93', Lat+'', Long+'', 0, I, XDest, YDest, uaDegre);
                Les_Fiches[Fiche_Courante].Localisation := tlMetropole;
              end
              else
              if copy(S, 1, 18)='fiche_point_OM.asp' then  // Outre-Mer
              begin  // conversion en UTM
                if (Site='9731101') or (Site='97353A') then  // deux sites en Guyane sont en zone UTM 21 mais il faut quand mme faire la conversion en UTM22
                  Donnees.Conversion('WGS84', 'UTM 22 N', Lat+'', Long+'', 0, I, XDest, YDest, uaDegre)
                else
                  Donnees.Conversion('WGS84', 'UTM', Lat+'', Long+'', 0, I, XDest, YDest, uaDegre);
                Les_Fiches[Fiche_Courante].Localisation := tlOutreMer;
              end
              else
                raise Exception.Create('Rgion inconnue');
              Les_Fiches[Fiche_Courante].X := copy(Xdest, 1, pos('.', XDest)-1 ); // supression des chiffres aprs la virgule
              Les_Fiches[Fiche_Courante].Y := copy(Ydest, 1, pos('.', YDest)-1 ); // supression des chiffres aprs la virgule

              writeln(OSM,format('  <node id=''-%d'' lat=''%s'' lon=''%s''>',[Point_Courant, Lat, Long]));
              writeln(OSM,'    <tag k=''man_made'' v=''survey_point'' />');
              writeln(OSM,format('    <tag k=''ref'' v=''%s - %s'' />',[Site, Indice]));
              if Remarque<>'' then
                Designation := Designation + ' ' + Remarque;
              if Designation<>'' then
                writeln(OSM,format('    <tag k=''description'' v=''%s'' />',[Designation]));
              if altitude<>'' then
              begin
                if Pos(',',Altitude)<>0 then
                  Altitude[pos(',',Altitude)]:='.';
                writeln(OSM,format('    <tag k=''ele'' v=''%s'' />',[Altitude]));
              end;
              writeln(OSM,'    <tag k=''source'' v=''©IGN 2010 dans le cadre de la cartographie réglementaire'' />');
              writeln(OSM,format('    <tag k=''url'' v=''%s%s'' />',[BaseURL,S]));
              writeln(OSM,'    <tag k=''note'' v=''Ne pas déplacer ce point, cf. - Do not move this node, see - http://wiki.openstreetmap.org/wiki/WikiProject_France/Repères_Géodésiques#Permanence_des_rep.C3.A8res'' />');
              writeln(OSM,'  </node>');
            end;
          end;
        finally
          CloseFile(G);
        end;
      finally
        CloseFile(F);
      end;

  // criture des relations correspondant aux fiches dans le fichier OSM
      for I:=0 to Nb_Fiche-1 do
       with Les_Fiches[I] do
        if Nb_Point>0 then
      begin
        writeln(OSM, Format('  <relation id=''-10%.5d''>',[I]));
        for J:=1 to Nb_Point do
          writeln(OSM, Format('    <member type=''node'' ref=''-%d'' />',[Points[J]]));
//          writeln(OSM, Format('    <member type=''node'' ref=''-%d'' role='''' />',[Points[J]]));
        if Numero<>'' then
          writeln(OSM,format('    <tag k=''ref'' v=''%s'' />',[Numero]));
        if Nom<>'' then
          writeln(OSM,format('    <tag k=''name'' v=''%s'' />',[Nom]));
        if Lieu_Dit<>'' then
          writeln(OSM,format('    <tag k=''note'' v=''%s'' />',[Lieu_Dit]));
        if Reseau<>'' then
          writeln(OSM,format('    <tag k=''network'' v=''%s'' />',[reseau]));
        writeln(OSM,'    <tag k=''type'' v=''site'' />');
        writeln(OSM,'    <tag k=''site'' v=''geodesic'' />');

        case Localisation of
          tlMetropole : writeln(OSM,format('    <tag k=''url'' v=''%sfiche_geodesie.asp?num_site=%s&amp;X=%s&amp;Y=%s'' />',[BaseURL, Numero, X, Y]));
          tlOutreMer  : writeln(OSM,format('    <tag k=''url'' v=''%sfiche_geodesie_OM.asp?num_site=%s&amp;X=%s&amp;Y=%s'' />',[BaseURL, Numero, X, Y]));
        end;  

        writeln(OSM,'    <tag k=''source'' v=''©IGN 2010 dans le cadre de la cartographie réglementaire'' />');
        writeln(OSM,'  </relation>');
      end;
      writeln(OSM,'</osm>');
    finally
      CloseFile(OSM);
    end;

  finally
    Donnees.Free;
  end;

  writeln('Termin');
  readln;
end.
